/*
 * SPDX-FileCopyrightText: 2023 g10 code GmbH
 * SPDX-Contributor: Carl Schwan <carl.schwan@gnupg.com>
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

body {
  font-family: "Segoe UI Web (West European)",Segoe UI,-apple-system,BlinkMacSystemFont,Roboto,Helvetica Neue,sans-serif;
}

.d-flex {
  display: flex;
  flex-direction: column;
}

.flex-row {
  flex-direction: row;
}

.sr-only {
  position: absolute;
  width: 1px;
  height: 1px;
  padding: 0;
  margin: -1px;
  overflow: hidden;
  clip: rect(0, 0, 0, 0);
  white-space: nowrap;
  border-width: 0;
}

.flex-row {
  flex-direction: row !important;
}

.d-none {
  display: none;
}

.w-100 {
  width: 100%;
}

.btn {
  background: none;
  border: none;
  padding: 0.5rem;
  text-align: left;
  background-color: rgb(235, 243, 252);
  display: flex;
  gap: 0.5rem;
  align-items: center;
}

.rounded-md {
  border-radius: 3px;
}

.rounded-s-md {
  border-bottom-left-radius: 3px;
  border-top-left-radius: 3px;
}

.rounded-e-md {
  border-bottom-right-radius: 3px;
  border-top-right-radius: 3px;
}

.btn:not([disabled]):hover {
  background-color: rgb(207, 228, 250);
}

.btn-danger {
  background-color: #f4a4a4;
}

.btn-danger:not([disabled]):hover {
  background-color: #fb8484;
}

.align-items-center {
  align-items: center;
}

.gap {
  gap: 0.5rem;
}

.mt-0, .my-0, .m-0 {
  margin-top: 0 !important;
}

.mb-0, .my-0, .m-0 {
  margin-bottom: 0 !important;
}

.mt-3, .my-3, .m-3 {
  margin-top: 1rem !important;
}

.mb-3, .my-3, .m-3 {
  margin-bottom: 1rem !important;
}

.pt-0, .py-0, .p-0 {
  padding-top: 0 !important;
}

.pb-0, .py-0, .p-0 {
  padding-bottom: 0 !important;
}

.ps-0, .px-0, .p-0 {
  padding-left: 0 !important;
}

.pe-0, .px-0, .p-0 {
  padding-right: 0 !important;
}

.pt-1, .py-1, .p-1 {
  padding-top: 0.25rem !important;
}

.pb-1, .py-1, .p-1 {
  padding-bottom: 0.25rem !important;
}

.ps-1, .px-1, .p-1 {
  padding-left: 0.25rem !important;
}

.pe-1, .px-1, .p-1 {
  padding-right: 0.25rem !important;
}

.pt-2, .py-2, .p-2 {
  padding-top: 0.5rem !important;
}

.pb-2, .py-2, .p-2 {
  padding-bottom: 0.5rem !important;
}

.ps-2, .px-2, .p-2 {
  padding-left: 0.5rem !important;
}

.pe-2, .px-2, .p-2 {
  padding-right: 0.5rem !important;
}

.pt-3, .py-3, .p-3 {
  padding-top: 1rem !important;
}

.pb-3, .py-3, .p-3 {
  padding-bottom: 1rem !important;
}

.ps-3, .px-3, .p-3 {
  padding-left: 1rem !important;
}

.pe-3, .px-3, .p-3 {
  padding-right: 1rem !important;
}

.ms-auto, .mx-auto {
  margin-left: auto !important;
}

.me-auto, .mx-auto {
  margin-right: auto !important;
}

.list-unstyled {
  padding-left: 0;
  list-style: none;
}

body {
  font-family: "Noto Sans";
}

h2 {
  font-size: 1.5rem;
}

.alert-error {
  background-color: #F294B2;
}

.alert-warning {
  background-color: #FCBC99;
}

hr {
  border-top: 1px black solid;
  border-bottom: 0px;
}

.w-32 {
  width: 32px !important;
}

.h-32 {
  height: 32px !important;
}
